﻿



local DSxFriendEventLib_MAJOR, DSxFriendEventLib_MINOR = "DSxFriendEventLib", 21
local DSxFriendEventLib, oldminor = LibStub:NewLibrary(DSxFriendEventLib_MAJOR, DSxFriendEventLib_MINOR)

if not DSxFriendEventLib then return end -- No upgrade needed

--
-- Private Section
--
local DSxReusableLib = LibStub("DSxReusableLib")
local AceEvent = LibStub('AceEvent-3.0')
local AceBucket = LibStub('AceBucket-3.0')
local DSxReusableLib = LibStub('DSxReusableLib')

local defaultRecord= {
    name=nil,
    level=0,
    class='Unknown',
    location = 'Unknown',
    connected = nil,
    status = nil,
    friendNote = '',
    createTime = 0,
    endTime = nil
}

local private = {}
private.lastScan={}
private.lastScanReusableRecords=DSxReusableLib:New(defaultRecord)
private.objects = {}

function private:FRIENDLIST_UPDATE()
    local curTime=time()
    
    for i=1,GetNumFriends() do
        local name, level, class, location, connected, status, friendNote = GetFriendInfo(i)
        
        if not self.lastScan[name] then
            if connected then
                local record = self.lastScanReusableRecords:Borrow()
                record.name=name
                record.level=level
                record.class=class
                record.location = location
                record.connected = connected
                record.status=status
                record.friendNote=friendNote
                record.createTime=curTime
                
                self.lastScan[name] = record
                
                --tell object of the new record
                for k,_ in pairs(self.objects) do
                    if k.OnFriendLogin then
                        k:OnFriendLogin(record)
                    end
                end
            end
        else
            local record = self.lastScan[name]
            
            if record.connected ~= connected then
                --tell object of the logout
                
                record.endTime=curTime
                for k,_ in pairs(self.objects) do
                    if k.OnFriendLogout then
                        k:OnFriendLogout(record)
                    end
                end
                
                self.lastScanReusableRecords:Return(record)
                self.lastScan[name]=nil;
            elseif connected and record.location and record.location ~= location then
                local newRecord = self.lastScanReusableRecords:Borrow()
                newRecord.name=name
                newRecord.level=level
                newRecord.class=class
                newRecord.location = location
                newRecord.connected = connected
                newRecord.status=status
                newRecord.friendNote=friendNote
                newRecord.createTime=curTime
                
                record.endTime=curTime
                
                --tell object of the new location
                for k,_ in pairs(self.objects) do
                    if k.OnFriendLocationUpdate then
                        k:OnFriendLocationUpdate(record, newRecord)
                    end
                end
                
                self.lastScanReusableRecords:Return(record)
                self.lastScan[name]=newRecord;
            end
        end
    end
end

--send fake logout messages for any users who are still online
function private:PLAYER_LOGOUT()
    for name, record in pairs(self.lastScan) do
        if record.connected then
            record.endTime=time()
        
            for k,_ in pairs(self.objects) do
                if k.OnFriendLogout then
                    k:OnFriendLogout(record)
                end
            end
        end
    end
end
--
-- Public Section
--


function DSxFriendEventLib:Embed(object)
    if not private.listening then
        private.listening=true
        AceBucket:Embed(private)
        private:RegisterBucketEvent({"FRIENDLIST_UPDATE"}, 1, "FRIENDLIST_UPDATE")

        AceEvent:Embed(private)
        private:RegisterEvent("PLAYER_LOGOUT", "PLAYER_LOGOUT")
    end

    private.objects[object]=1
    ShowFriends()
end